/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada. 
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.EventListener;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import ca.uvic.csr.shrimp.PanelModeConstants;
import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.VisVarConstants;
import ca.uvic.csr.shrimp.AttrToVisVarBean.AttrToVisVarBean;
import ca.uvic.csr.shrimp.AttrToVisVarBean.Attribute;
import ca.uvic.csr.shrimp.AttrToVisVarBean.NominalNodeColorVisualVariable;
import ca.uvic.csr.shrimp.DataBean.Artifact;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpArc;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.FilterBean.DegreeOfInterestFilter;
import ca.uvic.csr.shrimp.FilterBean.DegreeOfInterestHighlighter;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.FilterBean.InterestFilter;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.FilmStrip.ShrimpSnapShot;
import ca.uvic.csr.shrimp.gui.ShrimpView.AbstractShrimpViewFactory;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpViewFactory;
import ca.uvic.csr.shrimp.gui.quickview.QuickViewPanel;
import ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeArtifact;
import ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.ProtegeDataBean;
import ca.uvic.csr.shrimp.resource.ResourceHandler;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.SimpleInstance;
import edu.stanford.smi.protege.ui.DirectInstancesList;
import edu.stanford.smi.protege.ui.SubclassPane;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.ComponentFactory;
import edu.stanford.smi.protege.util.ComponentUtilities;
import edu.stanford.smi.protege.util.DoubleClickActionAdapter;
import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protege.util.LazyTreeModel;
import edu.stanford.smi.protege.util.LazyTreeNode;
import edu.stanford.smi.protege.util.LazyTreeRoot;
import edu.stanford.smi.protege.util.SelectableList;
import edu.stanford.smi.protege.util.SelectionEvent;
import edu.stanford.smi.protege.util.SelectionListener;

/**
 *  Creates all the Jambalaya UI Content.
 * 
 * @author Chris Callendar
 */
public class JambalayaContentPane {

	private IJambalayaComponent parent;
	private boolean showClsTree = true;
	
	// listeners for the instance list and class tree
	private UpdateShrimpViewOnInstanceSelection updateShrimpViewInstanceListener;
	private DegreeOfInterestChangeAdapter degreeOfInterestChangeAdapter;
	private SelectedNodesChangeAdapter selectedNodesChangeAdapter;

	// pane classes are displayed in
	private JSplitPane clsSplitPane;

	// pane instances are displayed in
	private JSplitPane instanceSplitPane;

	// other members used to create split panes
	//private ShrimpNode currentSelectedClsNode; // node that is currently selected in the classes panel
	private JambalayaClsesPanel clsesPanel;
	private JTree clsTree;
	private DirectInstancesList directInstancesList;
	private LabeledComponent instanceLabeledComponent;

	private JPanel shrimpViewContainer;

	public JambalayaContentPane(IJambalayaComponent parent) {
		this(parent, true);
	}
	
	public JambalayaContentPane(IJambalayaComponent parent, boolean showClsTree) {
		this.parent = parent;
		this.showClsTree = showClsTree;
		this.degreeOfInterestChangeAdapter = new DegreeOfInterestChangeAdapter();
	}
	
	public void setParent(IJambalayaComponent parent) {
		this.parent = parent;
	}
	
	public IJambalayaComponent getParent() {
		return parent;
	}
	
	private JambalayaProject getProject() {
		return JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaProject();
	}
	
	public void initialize() {
		// ensure that the application and project are created.
    	JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaApplication();
    	getProject();
    	
		
		// initialize class pane
		clsSplitPane = ComponentFactory.createLeftRightSplitPane();
		// initialize instances pane
		instanceSplitPane = ComponentFactory.createTopBottomSplitPane(false);

		clsSplitPane.setLeftComponent(createInstanceSplitter());
		shrimpViewContainer = new JPanel();
		shrimpViewContainer.setLayout(new BorderLayout());
		JLabel awtLabel = new JLabel("JAMBALAYA - Powered by", new ImageIcon (ResourceHandler.getResourceImage("shrimplogo.jpg")), SwingConstants.CENTER);
		awtLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		awtLabel.setVerticalTextPosition(SwingConstants.TOP);
		awtLabel.setForeground(Color.LIGHT_GRAY);
		shrimpViewContainer.setBackground(ShrimpConstants.SHRIMP_BACKGROUND_COLOR);
		shrimpViewContainer.add(awtLabel, BorderLayout.CENTER);
		clsSplitPane.setRightComponent(new JScrollPane(shrimpViewContainer));
		
		setSavedSplitterPositions();
	}

	public void setRootClses(Set rootClses) {
		// @tag Jambalaya(rootClses) : reload the root tree node with the root classes
		LazyTreeModel model = (LazyTreeModel) clsTree.getModel();
		LazyTreeRoot root = (LazyTreeRoot) model.getRoot();
		root.reload(rootClses);
		clsTree.setModel(model);
		clsTree.expandRow(0);
	}
	
	public Component getComponent() {
		if (showClsTree) {
			return clsSplitPane;
		}
		return shrimpViewContainer;
	}
	
	public JambalayaClsesPanel getClassesPanel() {
		return clsesPanel;
	}
	
	public LabeledComponent getLabeledComponent() {
		return clsesPanel.getLabeledComponent();
	}
	
	public LabeledComponent getInstancesLabeledComponent() {
		return instanceLabeledComponent;
	}
	
	public JTree getClsTree() {
		return clsTree;
	}
	
	public SelectableList getInstancesList() {
		return (SelectableList)directInstancesList.getSelectable();
	}
		
	private Object getBean(String beanName) throws BeanNotFoundException {
	    ShrimpProject activeProject = getProject();
	    if (activeProject == null) {
	        throw new BeanNotFoundException(beanName);
	    }
        return activeProject.getBean(beanName);
	}
	
	private Object getTool (String toolName) throws ShrimpToolNotFoundException {
	    ShrimpProject activeProject = getProject();
	    if (activeProject == null) {
	        throw new ShrimpToolNotFoundException (toolName);
	    }
        return activeProject.getTool(toolName);
	}
	
    /* (non-Javadoc)
     * @see edu.stanford.smi.protege.util.Disposable#dispose()
     */
    public void dispose() {
		//Save the position of the dividers in the APPLICATION properties
        JambalayaApplication app = JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaApplication();
		
        if (clsSplitPane != null) {
            app.getProperties().setProperty(JambalayaApplication.PROPERTY_KEY__CLS_SPLIT_PANE_LOCATION, "" + clsSplitPane.getDividerLocation());
        	clsSplitPane.removeAll();
        	clsSplitPane = null;
        }
        if (instanceSplitPane != null) {
            app.getProperties().setProperty(JambalayaApplication.PROPERTY_KEY__INSTANCE_SPLIT_PANE_LOCATION, "" + instanceSplitPane.getDividerLocation());
        	instanceSplitPane.removeAll();
        	instanceSplitPane = null;
        }
		//currentSelectedClsNode = null;
        if (clsesPanel != null) {
        	clsesPanel.clearSelection();
        	clsesPanel.removeAll();
        }
		if (clsTree != null) {
			clsTree.clearSelection();
			clsTree.removeAll();
			clsTree = null;
		}
		if (directInstancesList != null) {
			directInstancesList.clearSelection();
			directInstancesList.removeAll();
			directInstancesList = null;
		}
    }
    
	/**
	 * @param activeProject
	 */
	public ShrimpView createShrimpView(ShrimpProject activeProject) {
        ShrimpViewFactory factory = new JambalayaShrimpViewFactory();
        return factory.createShrimpView(activeProject, shrimpViewContainer);
	}

	// Create Instances split pane.
	private JComponent createInstanceSplitter() {
		instanceSplitPane.setLeftComponent(createClsesPanel());
		instanceSplitPane.setRightComponent(createInstancesPanel());

		return instanceSplitPane;
	}

	// Create Classes split pane.
	private JComponent createClsesPanel() {
		Project project = parent.getProject();
		
		// HACK to get the project name appearing correctly in the class panel
		// If the project URI is a jar file then the project name will be null 
		// because uri.getPath() returns null if it starts with 'jar:/'
		boolean changedURI = false;
		URI originalURI = project.getProjectURI(); 
		if (originalURI != null) {
			String uriStr = originalURI.toString();
			if (uriStr.startsWith("jar:")) {
				try {
					URI temp = new URI(uriStr.substring(4));
					project.setProjectURI(temp);
					changedURI = true;
				} catch (URISyntaxException e) {
				}
			}
		}
		
	    clsesPanel = new JambalayaClsesPanel(project);
	    
	    // now put the original URI back if it was changed
	    if (changedURI) {
			project.setProjectURI(originalURI);
	    }
	    
	    clsTree = (JTree)((SubclassPane) clsesPanel.getSelectable()).getSelectable();
		return clsesPanel;
	}

	// Create instances panel.
	private JComponent createInstancesPanel() {
		directInstancesList = new DirectInstancesList(parent.getProject());
		SelectableList list = (SelectableList) directInstancesList.getSelectable();

		// remove the instance list's default double click adapter
		EventListener[] listeners = list.getListeners(MouseListener.class);
		for (int i = 0; i < listeners.length; i++) {
			if (listeners[i] instanceof DoubleClickActionAdapter) {
				list.removeMouseListener((DoubleClickActionAdapter) listeners[i]);
			}
		}
		
		// get the LabeledComponent (it is a component in the instance list
		//@tag Protege(LabeledComponent): finding the labeled component in the instance list
		for (int i = 0; i < directInstancesList.getComponentCount(); i++) {
			Component c = directInstancesList.getComponent(i);
			if (c instanceof LabeledComponent) {
				instanceLabeledComponent = (LabeledComponent)c;
			}
		}		
		
		return directInstancesList;
	}

	private void addInstanceListListeners() {
		if (directInstancesList != null) {
			SelectableList list = (SelectableList) directInstancesList.getSelectable();
			updateShrimpViewInstanceListener = new UpdateShrimpViewOnInstanceSelection();
			list.addSelectionListener(updateShrimpViewInstanceListener);
			list.addMouseListener(new InstanceListDoubleClickAdapter());
			//list.addPropertyChangeListener(degreeOfInterestChangeAdapter);
		}
	}
	
	private void addClsTreeListeners() {
		if (clsTree != null) {
			clsTree.addTreeExpansionListener(new ClsTreeExpansionAdapter());
			clsTree.addTreeSelectionListener(new UpdateInstanceListAdapter());
			clsTree.addTreeSelectionListener(new UpdateShrimpViewOnClsSelectionAdapter());
			clsTree.addMouseListener(new ClsTreeMouseDoubleClickAdapter());
			clsTree.addPropertyChangeListener(degreeOfInterestChangeAdapter);
		}
	}
	
	private void setSavedSplitterPositions() {
		String classesDividerLocationStr = JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaApplication().getProperties().getProperty(JambalayaApplication.PROPERTY_KEY__CLS_SPLIT_PANE_LOCATION, "" + JambalayaApplication.PROPERTY_DEFAULT__CLS_SPLIT_PANE_LOCATION);
		int classesDividerLocation = Integer.parseInt(classesDividerLocationStr);
		if (clsSplitPane.getHeight() - classesDividerLocation < 50) {
		    classesDividerLocation = clsSplitPane.getHeight() - 50;
		}
		clsSplitPane.setDividerLocation(classesDividerLocation);
		String instanceDividerLocation = JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaApplication().getProperties().getProperty(JambalayaApplication.PROPERTY_KEY__INSTANCE_SPLIT_PANE_LOCATION, "" + JambalayaApplication.PROPERTY_DEFAULT__INSTANCE_SPLIT_PANE_LOCATION);
		instanceSplitPane.setDividerLocation(Integer.parseInt(instanceDividerLocation));
	}
	
	
	// navigate shrimp view to the chosen node
	private void navigateToNode(ShrimpNode node) {
		if (node == null) 
			return;
		
		ShrimpProject shrimpProject = JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaProject();
		if (shrimpProject == null) return;
		try {
			ShrimpView shrimpView = (ShrimpView) shrimpProject.getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);

			Vector currentObjects = (Vector) displayBean.getCurrentFocusedOnObjects().clone();
			Object srcObject;
			if (currentObjects.size() == 0 || currentObjects.elementAt(0) instanceof ShrimpArc) {
			    srcObject = displayBean.getDataDisplayBridge().getRootNodes();
			} else {
			    srcObject = currentObjects;
			}

			if (!srcObject.equals(node)) {
				Vector path = displayBean.getPathBetweenObjects(srcObject, node);
				if (!path.isEmpty()) {
					shrimpView.navigateToObject(node);
				} else {
					JOptionPane.showMessageDialog(JambalayaProjectPlugin.getJambalayaProjectPlugin().getJambalayaApplication().getParentFrame(), "The destination node is not in current hierarchy.");
				}
			}
		} catch (ShrimpToolNotFoundException e) {
			// do nothing
		} catch (BeanNotFoundException e) {
			// do nothing
		}
	}
	

	/**
	 * Gets the artifacts for the given frames using the databean.
	 * @param c A collection of Frame objects
	 * @return Vector of Artifact objects
	 */
	protected Vector getArtifactsForFrames(Collection frames) {
		Vector artifacts = new Vector();
		try {
			ProtegeDataBean dataBean = (ProtegeDataBean) getBean(ShrimpProject.DATA_BEAN);
			for (Iterator iter = frames.iterator(); iter.hasNext(); ) {
				Frame frame = (Frame) iter.next();
				Artifact artifact = dataBean.findArtifact(frame);
				if (artifact != null) {
					artifacts.add(artifact);
				}
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}		
		return artifacts;
	}
	

	private Vector getChosenNodes(TreePath[] treePaths) {
		Vector chosenNodes = new Vector();
		for (int i = 0; i < treePaths.length; i++) {
			TreePath treePath = treePaths[i];
			ShrimpNode chosenNode = getChosenNode(treePath);
			if (chosenNode != null) {
				chosenNodes.add(chosenNode);
			}
		}
		return chosenNodes;
	}

	private ShrimpNode getChosenNode(TreePath treePath) {
		// find the chosen tree node
		LazyTreeNode treeNode = (LazyTreeNode) treePath.getLastPathComponent();
		Frame frame = (Frame) treeNode.getUserObject();
		List keyParents = createKeyParentListForCls(treePath);
		return getChosenNode(frame, keyParents);
	}

	private ShrimpNode getChosenNode(Frame frame, List keyParents) {
		ShrimpNode chosenNode = null;
		try {
			ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
			DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
			ProtegeDataBean dataBean = (ProtegeDataBean) getBean(ShrimpProject.DATA_BEAN);

			Artifact chosenArtifact = dataBean.findArtifact(frame);

			if (chosenArtifact != null) {
				boolean findProperTreePath = (frame instanceof SimpleInstance && isHierarchy(Model.Slot.DIRECT_INSTANCES) && isHierarchy(Model.Slot.DIRECT_SUBCLASSES)) || (frame instanceof Cls && isHierarchy(Model.Slot.DIRECT_SUBCLASSES));
				if (findProperTreePath && !keyParents.isEmpty()) {
					chosenNode = displayBean.getDataDisplayBridge().getShrimpNode(chosenArtifact, keyParents, true);
				} else {
					Vector allNodesRepresentingChosenArtifact = displayBean.getDataDisplayBridge().getShrimpNodes(chosenArtifact, true);
					// just use the first node in the list
					if (!allNodesRepresentingChosenArtifact.isEmpty())
						chosenNode = (ShrimpNode) allNodesRepresentingChosenArtifact.firstElement();
				}
			} 
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}

		return chosenNode;
	}
	
	private boolean isHierarchy (String cprel) {
		boolean b = false;
		try {
			ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
			Set currentCprels = new HashSet(Arrays.asList( shrimpView.getCprels()));
			b = currentCprels.contains(cprel);
		} catch (ShrimpToolNotFoundException e) {
			e.printStackTrace();
		} 
		return b;	
	}
	
	private List createKeyParentListForInstance(TreePath treePath) {
		return createKeyParentList(treePath, 1);
	}

	private List createKeyParentListForCls(TreePath treePath) {
		return createKeyParentList(treePath, 2);
	}

	private List createKeyParentList(TreePath treePath, int offset) {
		List keyParentsList = new ArrayList();
		// travel up the tree adding key parents to the list
		for (int i = treePath.getPathCount() - offset; i > -1; i--) {
			LazyTreeNode treeNode = (LazyTreeNode) treePath.getPathComponent(i);
			Object object = treeNode.getUserObject();
			if (object instanceof Cls) {
				try {
					ProtegeDataBean dataBean = (ProtegeDataBean) getBean(ShrimpProject.DATA_BEAN);

					Cls cls = (Cls) object;

					Artifact keyParent = dataBean.findArtifact(cls);
					if (keyParent != null) {
						keyParentsList.add(keyParent.getExternalId());
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return keyParentsList;
	}


	/**
	 * Sets the interest filter.  This is used to determine which nodes are interesting.
	 * @param filter
	 */
	public void setInterestFilter(InterestFilter filter) {
		degreeOfInterestChangeAdapter.setFilter(new DegreeOfInterestFilter(filter));
		degreeOfInterestChangeAdapter.setHighlighter(new DegreeOfInterestHighlighter(filter));
	}
	
	/**
	 * Takes a snapshot
	 * @param project
	 * @param shrimpView
	 */
	public void takeSnapShot(ShrimpProject project) {
		try {
			ShrimpSnapShot.takeSnapShot(project, (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW));
		} catch (ShrimpToolNotFoundException ignore) {
		}
	}

	// when node in the class tree is expanded, show children of corresponding node in shrimp view
	// when node in the class tree is collapsed, close corresponding node in shrimp view
	private class ClsTreeExpansionAdapter implements TreeExpansionListener {
		public void treeExpanded(TreeExpansionEvent event) {
			//System.out.println("ClsTreeExpansionAdapter.treeExpanded");					
			ShrimpNode node = getChosenNode(event.getPath());
			if (node != null) {
				try {
					ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
					DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
					displayBean.setPanelMode(node, PanelModeConstants.CHILDREN);
				} catch (ShrimpToolNotFoundException e) {
					// do nothing
				} catch (BeanNotFoundException e) {
					// do nothing
				}
			}
		}

		public void treeCollapsed(TreeExpansionEvent event) {
			//System.out.println("ClsTreeExpansionAdapter.treeCollapsed");					
			ShrimpNode node = getChosenNode(event.getPath());
			if (node != null) {
				try {
					ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
					DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
					displayBean.setPanelMode(node, PanelModeConstants.CLOSED);
				} catch (ShrimpToolNotFoundException e) {
					// do nothing
				} catch (BeanNotFoundException e) {
					// do nothing
				}
			}
		}
	}

	private class UpdateInstanceListAdapter implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent event) {
			// update instances list
			Collection selection = ComponentUtilities.getSelection(clsTree);
			try {
				// first instance selected automatically when calling setClses, which we don't want
				// so we temporarily remove our listener for selection of instances, and set the selection to nothing
				SelectableList list = (SelectableList) directInstancesList.getSelectable();
				updateShrimpViewInstanceListener.disable();
				directInstancesList.setClses(selection);
				list.clearSelection();
				updateShrimpViewInstanceListener.enable();
			} catch (IndexOutOfBoundsException e) {
				// do nothing with this for now, I think this may be Protege's problem? Rob Jan 9, 2003
			}
		}
	}

	private class UpdateShrimpViewOnClsSelectionAdapter implements TreeSelectionListener {
		private boolean enabled = true;

		public void disable() {
			enabled = false;
		}

		public void enable() {
			enabled = true;
		}

		public void valueChanged(TreeSelectionEvent event) {
			if (!enabled)
				return;

			TreePath[] treePaths = clsTree.getSelectionPaths();
			if (treePaths == null)
				return;

			try {
				// get the ShrimpNodes for the classes that have been selected
				Vector chosenNodes = getChosenNodes(treePaths);
				// this returns all the selected frames
				//Collection c = ComponentUtilities.getSelection(clsTree, Frame.class);
				//Vector chosenArtifacts = getArtifactsForFrames(c);
				ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectedNodesChangeAdapter.disable();
				selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, chosenNodes);
				selectedNodesChangeAdapter.enable();
				//System.out.println(chosenNodes);
			} catch (Exception ignore) {
			}
		}
	}

	// when a node in the class tree is double clicked, navigate shrimp view to that node
	private class ClsTreeMouseDoubleClickAdapter extends MouseAdapter {
		public void mouseClicked(MouseEvent event) {
			if (event.getClickCount() == 2 && SwingUtilities.isLeftMouseButton(event) && clsTree.getSelectionCount() == 1) {
				ShrimpNode node = getChosenNode(clsTree.getSelectionPath());
				if (node != null) {
					navigateToNode(node);
				}
			}
		}
	}

	private class UpdateShrimpViewOnInstanceSelection implements SelectionListener {
		private boolean enabled = true;

		public void disable() {
			enabled = false;
		}

		public void enable() {
			enabled = true;
		}

		public void selectionChanged(SelectionEvent event) {

			if (!enabled) {
				return;
			}
			
			if (clsTree == null) {
				return;
			}

			TreePath[] treePaths = clsTree.getSelectionPaths();

			// if no classes are selected, there should be no instances selected either
			if (treePaths == null) {
				return;
			}

			Vector instances = new Vector(directInstancesList.getSelection());

			Vector chosenNodes = new Vector();
			for (int i = 0; i < treePaths.length; i++) {
				TreePath treePath = treePaths[i];

				// get the ShrimpNodes for all the selected classes
				List keyParents = createKeyParentListForCls(treePath);
				Cls cls = (Cls) ((LazyTreeNode) treePath.getLastPathComponent()).getUserObject();

				// this key parents vector will be same for all the instances 
				// of the class that this tree path belongs to
				keyParents = createKeyParentListForInstance(treePath);

				// now get the nodes for the instances that belong to this class
				for (int j = 0; j < instances.size(); j++) {
					Instance instance = (Instance) instances.get(j);

					if (cls.getDirectInstances().contains(instance)) {
						ShrimpNode node = getChosenNode(instance, keyParents);
						if (node != null)
							chosenNodes.add(node);

						instances.remove(j);
						j--;
					}
				}
			}

			if (chosenNodes.size() > 0) {
				try {
					ShrimpView shrimpView = (ShrimpView) getTool(ShrimpProject.SHRIMP_VIEW);
					SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);

					selectedNodesChangeAdapter.disable();
					selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, chosenNodes);
					selectedNodesChangeAdapter.enable();
				} catch (ShrimpToolNotFoundException e) {
					// do nothing
				} catch (BeanNotFoundException e) {
					// do nothing
				}
			}
		}
	}

	private class InstanceListDoubleClickAdapter extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 2 && SwingUtilities.isLeftMouseButton(e)) {

				TreePath[] treePaths = clsTree.getSelectionPaths();

				// if no classes are selected, there should be no instances selected either
				if (treePaths == null) {
					return;
				}

				Instance instance = (Instance) CollectionUtilities.getFirstItem(directInstancesList.getSelection());

				// get the ShrimpNode for the instance
				ShrimpNode chosenNode = null;
				for (int i = 0; i < treePaths.length; i++) {
					TreePath treePath = treePaths[i];

					List keyParents = createKeyParentListForCls(treePath);
					Cls cls = (Cls) ((LazyTreeNode) treePath.getLastPathComponent()).getUserObject();

					if (cls.getDirectInstances().contains(instance)) {
						keyParents = createKeyParentListForInstance(treePath);
						chosenNode = getChosenNode(instance, keyParents);

						break;
					}
				}

				if (chosenNode != null) {
					navigateToNode(chosenNode);
				}
			}
		}
	}

	/*  
	 * listens for changes to "selected nodes" in the shrimp view 
	 * and selects the appropriate class or instance in the class tree and/or instance list
	 */
	class SelectedNodesChangeAdapter implements PropertyChangeListener {
		private boolean enabled = true;

		public void disable() {
			enabled = false;
		}

		public void enable() {
			enabled = true;
		}

	    public void setSelectedClses(Collection clses) {
	    	LazyTreeNode root = (LazyTreeNode) clsTree.getModel().getRoot();
	    	Collection rootClses = null;
	    	if (root.getUserObject() instanceof Collection) {
				rootClses = (Collection) root.getUserObject();
			}
	    	
	        Collection paths = new ArrayList();
	        Iterator i = clses.iterator();
	        while (i.hasNext()) {
	            Cls cls = (Cls) i.next();
	            paths.add(getPathToRoot(cls, new LinkedList(), rootClses));
	        }
	        ComponentUtilities.setSelectedObjectPaths(clsTree, paths);
	    }

	    private List getPathToRoot(Cls cls, LinkedList list, Collection rootClses) {
	        list.add(0, cls);
	        Cls rootCls = cls.getKnowledgeBase().getRootCls();
	        if (rootClses == null) {
	        	rootClses = new HashSet(1);
	        	rootClses.add(rootCls);
	        }
	        
	        // check if this cls is one of our root classes
	        if (rootClses.contains(cls)) {
        		return list;
	        }
	        
	        Iterator i = cls.getDirectSuperclasses().iterator();
	        while (i.hasNext()) {
	            Cls superclass = (Cls) i.next();
	            if (cls.isVisible()) {
	                List copy = new ArrayList(list);
	                getPathToRoot(superclass, list, rootClses);
	    	        // check if the super class is one of our root classes
	    	        if (rootClses.contains(list.getFirst())) {
	            		break;
	    	        }

	                // Backtracking
	                list.clear();
	                list.addAll(copy);
	            }
	        }
	        return list;
	    }	    

		public void propertyChange(PropertyChangeEvent evt) {
		    if (!enabled) 
		    	return;
		    
			Vector selectedNodes = (Vector) evt.getNewValue();
			Vector clses = new Vector();	// Cls objects
			Vector instances = new Vector(); // SimpleInstances selected

			for (Iterator selectedNodeIter = selectedNodes.iterator(); selectedNodeIter.hasNext();) {
				ShrimpNode selectedNode = (ShrimpNode) selectedNodeIter.next();
				Artifact selectedArtifact = selectedNode.getArtifact();
				Frame selectedFrame = ((ProtegeArtifact) selectedArtifact).getFrame();
				if (selectedFrame instanceof SimpleInstance) {
					instances.add(selectedFrame);
				} else if (selectedFrame instanceof Cls) {
					clses.add(selectedFrame);
				}
			}
			
			if (clses.size() > 0) {
				setSelectedClses(clses);
			}
			if (instances.size() > 0) {
				ComponentUtilities.setSelectedValues((SelectableList)directInstancesList.getSelectable(), instances);
			}
		}
	}

	/**
	 * Listens for filter and highlight property changes events.  These get fired
	 * from the class tree or the instance list when either are filtered or highlighted.
	 * The Diamond plugin for Protege will fire these events.
	 * Adds or removes a visibility Filter to filter invisible nodes.
	 * @see VisibilityFilter
	 * @author Chris Callendar
	 */
	private class DegreeOfInterestChangeAdapter implements PropertyChangeListener {
		
		/** The property change content type. */
		private static final String FILTER_PROPERTY_CHANGE = "FilterPropertyChange";
		private static final String HIGHLIGHT_PROPERTY_CHANGE = "HighlightPropertyChange";
		
		private DegreeOfInterestFilter filter = null;
		private DegreeOfInterestHighlighter highlighter = null;
		
		public void setFilter(DegreeOfInterestFilter filter) {
			this.filter = filter;
		}
		
		public void setHighlighter(DegreeOfInterestHighlighter highlighter) {
			this.highlighter = highlighter;
		}
		
		public void propertyChange(PropertyChangeEvent evt) {
			if (shrimpViewContainer == null)
				return;
			
			boolean filterOn = ((evt.getNewValue() instanceof Boolean) && 
								((Boolean)evt.getNewValue()).booleanValue());
			
			// add the filter to remove invisible nodes
			if (FILTER_PROPERTY_CHANGE.equals(evt.getPropertyName())) {
				if (filter == null)
					return;
				
				//System.out.println("FILTER PROPERTY CHANGE " + filterOn);
				try {
					ShrimpView shrimpView = (ShrimpView)getTool(ShrimpProject.SHRIMP_VIEW);
					FilterBean filterBean = (FilterBean)shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
					if (filterOn) {
						if (filterBean.contains(filter)) {
							filterBean.removeFilter(filter);
						}
						filterBean.addFilter(filter);
					} else if (filterBean.contains(filter)) {
						filterBean.removeFilter(filter);
						//updateNodeColors();
					}
				} catch (Exception e) {
					//e.printStackTrace();
				}
			} else if (HIGHLIGHT_PROPERTY_CHANGE.equals(evt.getPropertyName())) {
				if (highlighter == null)
					return;
				
				//System.out.println("HIGHLIGHT PROPERTY CHANGE " + filterOn);
				try {
					ShrimpView shrimpView = (ShrimpView)getTool(ShrimpProject.SHRIMP_VIEW);
					FilterBean filterBean = (FilterBean)shrimpView.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);
					highlighter.setApplied(filterOn);
					// we have to remove this filter and then add it back to apply the new highlighting
					if (filterBean.contains(highlighter)) {
						filterBean.removeFilter(highlighter);
					}
					filterBean.addFilter(highlighter);
					updateNodeColors();
				} catch (Exception e) {
					//e.printStackTrace();
				}
			}
		}
		
		/**
		 * Forces all the node colors to be updated.  The display is also repainted.
		 */
		private void updateNodeColors() {
	        try {
	        	// update the node colors
				AttrToVisVarBean attrToVisVarBean = (AttrToVisVarBean) getProject().getBean(ShrimpProject.ATTR_TO_VIS_VAR_BEAN);
				NominalNodeColorVisualVariable visVar = new NominalNodeColorVisualVariable(attrToVisVarBean, VisVarConstants.VIS_VAR_NODE_COLOR);
				Attribute attr = attrToVisVarBean.getMappedAttribute(VisVarConstants.VIS_VAR_NODE_COLOR); 
				attrToVisVarBean.fireVisVarValuesChangeEvent(attr, visVar);
			} catch (Exception e) {
				e.printStackTrace();
			}
			refreshDisplay();
		}

		/**
		 * Gets the shrimp view for the active project and repaints the display.
		 */
		private void refreshDisplay() {
	        try {
				ShrimpView shrimpView = (ShrimpView) getProject().getTool(ShrimpProject.SHRIMP_VIEW);
				DisplayBean displayBean = (DisplayBean) shrimpView.getBean(ShrimpTool.DISPLAY_BEAN);
				//shrimpView.clear();
				displayBean.repaint();
			} catch (Exception e) {
				e.printStackTrace();
			}	
		}		
		
	}
	
	public class JambalayaShrimpViewFactory extends AbstractShrimpViewFactory {

		protected void projectSpecificDisposeShrimpView() {
		}

		/**
		 * @see ca.uvic.csr.shrimp.gui.ShrimpView.AbstractShrimpViewFactory#projectSpecificCreateShrimpView()
		 */
		protected void projectSpecificCreateShrimpView() {
			// Add a border around display bean to tell whether it is focused or not
			shrimpViewContainer.removeAll();
			shrimpViewContainer.add(shrimpView.getGUI(), BorderLayout.CENTER);
			
			//create a panel of quickview buttons along the left hand side
//			Collection actions = new ArrayList();
//			String [] actionNames = PROTEGE_QUICK_VIEW_ACTION_NAMES;
//			for (int i = 0; i < actionNames.length; i++) {
//                String actionName = actionNames[i];
//				Action action = project.getActionManager().getAction(actionName, ShrimpConstants.MENU_QUICK_VIEWS);
//				if (action != null) {
//				    actions.add(action);
//				}
//            }
//			// add the Query View action
//			Action action = project.getActionManager().getAction(JambalayaApplication.QUICK_VIEW_QUERY_VIEW, ShrimpConstants.MENU_QUICK_VIEWS);
//			if (action != null) {
//			    actions.add(action);
//			}
			
			QuickViewPanel pnlQuickViews = new QuickViewPanel(project, new JambalayaTransferHandler(project));
			shrimpViewContainer.add(pnlQuickViews.getQuickViewPanel(), BorderLayout.WEST);
			
			//Validate now so that we can get acquire the size of the panel to 
			//properly set the root size
			parent.validate();
			shrimpViewContainer.setVisible(true);
			
			// @tag Shrimp(Jambalaya(Synch_Selection)): keep selection synched between class tree and Jambalaya view
			try {
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectedNodesChangeAdapter = new SelectedNodesChangeAdapter();
				selectorBean.addPropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, selectedNodesChangeAdapter);
			} catch (BeanNotFoundException e2) {
				e2.printStackTrace();
			}

			addClsTreeListeners();
			addInstanceListListeners();
			
		}
		
		protected void takeClosingSnapShot(ShrimpProject project) {
			Component gui = shrimpView.getGUI();
			// don't bother taking a snapshot if the gui has been disposed already
			if ((gui != null) && gui.isDisplayable()) {
				takeSnapShot(project);
			}
		}
	}


}
